/*
    CUSTDATA.CC
    Gary White
    10/06/1998

    Custom database object that enhances the emptyTable 
    method to reset any autoincrement fields in the table.
    
    Note it has not been completely and thoroughly tested,
    so back up any important tables before using it.  Use
    at your own risk.
    
    Comments, questions, suggestions to gary@apptools.com
    Complaints to bgates@microsoft.com

    Table structure info is based on hacking by Jim Sare,
    Jay Parsons, and a little by myself.

*/
class custdata of database

   function emptyTable( cTableName )
   local e, t, i, f, nOffset, cFullName
   try
   	// First call the built-in emptyTable routine to
   	// actually pack the table.
      super::emptyTable( cTableName )
   catch ( exception e )
   	// If it fails, tell the user
      showError( e.message )
      return	// and exit the routine
   endtry

	// grab the table info
   t=new tableDef()
   t.database = this
   t.tableName = cTableName
   
   try
	   t.load()
   catch ( exception e )
   	// should never fail, but...
      showError( e.message )
      return
   endtry

	// now just check to make sure we're not dealing with
	// something on which this won't work.
   if ( t.tableType # "DBASE" ) or ( t.version < 7 )
   	showError( "Resetting AutoInc's only works on dBASE level 7 tables" )
   	// and exit
      return
   endif

	// if the param did not include the file extension
   if upper( right( cTableName, 4 ) ) # ".DBF"
   	// add it.
   	cTableName += ".DBF"
   endif

	// now let's prepend the path to the filename
	cFullName = getDatabaseDir( this ) + "\" + cTableName

	// now let's see if we have any autoinc's
   for i = 1 to t.fields.size
   	if t.fields[i].type == "AUTOINC"
      	// we'll use some low level file stuff here
      	f=new file()
         try
         	// attempt to open the file
            f.open( cFullName, "RW")
         catch ( exception e )
         	// let user know if it didn't work
            showError( "Failed to open table to reset autoinc:" + ;
            	chr( 13 ) + e.message )
            return	// and exit
         endtry

         // NOTE: Structure assumptions are based on hacks, are NOT 
         // official and could change with future versions.
         //
         // Now calculate the offset for the autoinc's next value.
         // The field descriptor array begins at byte 68.  Each 
         // field descriptor is 48 bytes.  The next value of an 
         // AutoInc is stored as 4 bytes (least significant first) 
         // beginning 42 bytes after the start of the field 
         // descriptor.
         nOffset = ( 68 + ( 48 * ( i - 1 ) ) + 42 )

         // Try to seek the proper offset
         if f.seek( nOffSet, 0 ) <> nOffset
         	// if we fail
            f.close()		// close the file
            showError( "Low level seek failed" )	// tell the user
            return			// and exit
         endif
         // Here, we'll write the new value.  We want to reset
         // so the next value will be one.  So we write a one and
         // three zeros.
         if f.write( chr( 1 ) + replicate(chr(0), 3) ) # 4
         	// If the write function doesn't tell us it wrote 4 bytes
            // let the user know.  In all likelyhood, the table
            // structure is corrutpted if we land here.
         	showError( "Write failed to reset autoinc value" )
         endif
         // Now just close the file and return
         f.close()
      endif
	endfor
   return
endclass

//	Simple routine to display an error message
function showError( cMessage )
return msgbox( cMessage, "Error", 16 )

// Uses the BDE API to get the directory where the database lives
// by Romain Strieff
function GetDatabaseDir(oDatabase)
	// prototype the bde function
   if type("GetAliasPath") # "FP"
	   extern cint GetAliasPath(cHandle,cString,;
         cString,cString);
   		IDAPI32 from "DbiFormFullName"
   endif
   cRet=space(261)	// initialize a variable to hold the directory
   if ( GetAliasPath(oDatabase.handle,"YabbaDabbaDoo","",cRet ) == 0 )
   	// if we got it, trim the trailing backslash if it's there
      cRet=cRet.left(cRet.lefttrim().lastindexof("\"))
   else
   	// otherwise return an empty string
      cRet=""
   endif
RETURN cRet

