/*
   ----------------------------------------------------------
   Program...: CreateErrorTable.prg
   Programmer: Romain Strieff, Ken Mayer
   Date......: January 25, 1999
   Updated...: February 5, 2001
               Added errorFrom field, which notes
               if the error is from dBASE or the
               BDE ...

   Notes: this is based on a posting by Romain in the Visual
   dBASE newsgroups of a routine to create a text file
   containing error codes and error messages.

   However, most developers are more likely to want these
   in a table, so that's what we're doing here ...

   This deletes the table if it exists, and re-creates it.
   It stores the error number in the first field, and the
   error message in the second field, and creates
   an index on the errorNumber field so that a developer
   can then use the OODML findKey() or the XDML SEEK
   or Seek() to find the error in the table.

   It does require a BDE API call (DBIGetErrorMessage)
   to get the BDE Errors, but this should include
   everything ...

   Once the error table has been created, you can use
   it to look up errors by number to return the error
   message. This may be useful in some circumstances,
   although I find that for the most part it is useful
   when designing an error trap, to know what error is what.
   You can then look it up, and then decide what to do
   about that particular error in your try/catch code.
   ----------------------------------------------------------
*/

clear // the output pane of the Command Window

// If the table exists, dump it, so we can
// start fresh:
if file( "dBASEErrors.dbf" )
   drop table dBASEErrors
endif

// Create the table:
create table "dBASEErrors.dbf" ( errorFrom character( 5 ),;
                                errorNumber  numeric( 5 ),;
                                errorMessage character( 128 ) )

// Create index:
i = new DBFIndex()
i.indexName  := "errorNumber"
i.expression := "errorNumber"
_app.databases[1].createIndex( "dBASEErrors", i )

// Open the table:
q = new Query()
q.sql    := "select * from dBASEErrors"
q.active := true
r = q.rowset

? "Getting dBASE Errors"

// dBASE Error Messages 
// (uses the "message()" function):
for i = 1 to 401
    r.beginAppend()                 // add new row
    r.fields[1].value := "dBASE"
    r.fields[2].value := i          // error number
    r.fields[3].value := message(i) // error message
    r.save()                        // save it
next

// Extern the DbiGetErrorString BDE API function:
if type( "DbiGetErrorString" ) # "FP"
   extern cInt DbiGetErrorString( cInt, cString ) IDAPI32
endif

? "Getting BDE Errors"

// BDE Error Messages:
for i = 8000 to 14000
    cBuffer = space(128)
    DbiGetErrorString( i, cBuffer )

    // not all of these numbers
    // between 8000 and 14000 
    // actually have errors attached
    if not empty( cBuffer ) 
       r.beginAppend()
       r.fields[1].value := "BDE"
       r.fields[2].value := i
       r.fields[3].value := cBuffer
       r.save()
    endif
next

// deactivate the query:
q.active := false

? "-------------------------------"
? "Done creating table dBASEErrors."

/*
   ----------------------------------------------------------
   End of Program: CreateErrorTable.prg
   ----------------------------------------------------------
*/

