// Form Metric must be Pixels (6) for this
/*
    DateCalendar.cc
    Ken Mayer
    September, 2005

    Notes.....: This class is a container, which simply contains
                an entryfield and a pushbutton. The purpose is
                to datalink the entryfield as normal to a date
                field, and when the user runs a form with this
                on it, to load a calendar, directly under the 
                entryfield. 

    Usage.....: set procedure to :dUFLP:DateCalendar.cc additive
                // in the form designer, place this control
                // on a form. Set the datalink property of the
                // DateEntryfield in the container to whatever
                // field you need.
                //
                // You can also change the image used by the 
                // pushbutton easily enough ...
                //
                // In the form's onOpen or other event,
                // you might want to set the language, if you
                // want to use a language other than English:
                // 
                //    form.DateCalendar1.language := "Spanish" // for example
                //    Language options:
                //         English (default)
                //         Spanish
                //         French
                //         German
                //         Italian
                // Any other properties (color of the controls on
                // the calendar, etc.) are defined in the calendar
                // class in the dUFLP.
                // 
                // You can set the background color of the
                // drop down window:
                //   // colors:
                //   oMyDateCal.WindowColor = "0x80ffff"


    Dependencies:
                :dUFLP:PixCalendar22.cc
                :dUFLP:Center32.prg
                :dUFLP:WinAttr32.cc
                :dUFLP:WinPosition.cc
                :dUFLP:Calendar.bmp
*/

class DateCalendar( oParent ) of CONTAINER( oParent ) custom

   with (this)
      onOpen = class::DATECALENDAR_ONOPEN
      metric = 6 // Pixels
      left = 5.0
      top = 10.0
      width = 106.0
      height = 24.0
      borderStyle = 3 // None
      transparent = true
   endwith

   // custom properties:
   this.Language = "English" // "English", "French", "German", 
                             //"Spanish" or "Italian"
   this.dropped  = false     // whether the calendar form has "dropped" ...
   this.WindowColor = "Infobk"

   this.DATEENTRYFIELD = new ENTRYFIELD(this)
   with (this.DATEENTRYFIELD)
      height = 22.0
      left = 1.0
      top = 1.0
      width = 77.0
      value = {  /  /    }
      pageno = 0
      if smallFonts()
         fontSize = 10
      else
         fontSize = 8
      endif
   endwith

   this.GETCALBUTTON = new PUSHBUTTON(this)
   with (this.GETCALBUTTON)
      onClick = class::GETCALBUTTON_ONCLICK
      height = 24.0
      left = 80.0
      top = 0.0
      width = 25.0
      text = ""
      speedBar = true
      upBitmap = "filename:2 :dUFLP:Calendar.bmp"
      pageno = 0
   endwith


   function DATECALENDAR_onOpen
      // determine if button should have SystemTheme property set:
      if __version__ = 2.6
         // for these toolbuttons, don't use the system theme ...
         this.GetCalButton.systemTheme := false // for dBASE Plus 2.6
      endif

      // Defaults:
      this.dropped = false

      // Need WinPosition class to get positioning
      // information:
      set procedure to :dUFLP:WinPosition.cc additive
      oRef = new winPosition()
      oRef.get( int( this.DATEENTRYFIELD.hwnd ) )

      // Create instance of form, set top/left properties
      this.oCalForm = new CalendarForm()
      this.oCalForm.mdi = false
      this.oCalForm.top = oRef.bottom
      this.oCalForm.left = oRef.left
      this.oCalForm.parent = this // set parent for form

      // cleanup of winPosition
      oRef.release()
      oRef := null
      close procedure :dUFLP:WinPosition.cc

      // set colors
      this.oCalForm.colorNormal = this.WindowColor
   return

   function GETCALBUTTON_onClick
      local oCalForm, dCurrent
      private oDL
      dCurrent = this.parent.DateEntryfield.value

      // Before we go any further, let's see if we have
      // a dataLink:
      oDL = this.parent.DateEntryfield.dataLink
      if type( "oDL" ) == "O"
         // we have a datalink, let's see if autoEdit is false,
         // and the state is not edit or append ... (the user
         // in those cases needs to do something to go into
         // edit/append mode ...):
         if ( oDL.parent.parent.autoEdit == false ) and ;
            ( oDL.parent.parent.state # 2 and oDL.parent.parent.state # 3 )
            // don't drop the calendar form ...
            return
         endif
         // if the query object's requestLive property is false,
         // don't do anything ... can't edit the data:
         if oDL.parent.parent.parent.requestLive == false
            return
         endif
      endif
      // --------------------------------------------------------
      // if we hit here, the user should be able to edit
      // the data, or there is no dataLink in the first place ...
      // --------------------------------------------------------
      /*
         This should display a form directly under the entryfield 
         with a calendar on the form ...
      */
      if not this.parent.dropped
         this.parent.dropped := true
         this.parent.oCalForm.open()
      else
         this.parent.dropped := false
         this.parent.oCalForm.onLostFocus()
      endif
   return

endclass
//
// CalendarForm here:
// 
class CalendarForm of FORM
   with (this)
      onLostFocus = class::FORM_ONLOSTFOCUS
      onOpen = class::FORM_ONOPEN
      onClose = class::FORM_ONCLOSE
      open = class::FORM_OPEN
      scaleFontBold = false
      metric = 6	// Pixels
      height = 165.00
      width = 151.00
      text = ""
      topMost = true
      smallTitle = true
      sizeable = false
      moveable = false
      sysMenu = false
      escExit = false
      maximize = false
      minimize = false
      colorNormal = "Infobk"
   endwith

   this.OKButton = new Pushbutton( this )
   with( this.OKButton )
      onClick = class::OKClicked
      height = 20.0
      top = 144.0
      left = 19.0
      width = 118.0
      text = "Select Date"
      if smallFonts()
         fontSize = 8.0
      else
         fontSize = 7
      endif
   endwith

   function form_open
      class::Init()
   return FORM::open()

   function Init
      /*
         Set up the calendar class, and set the position,
         etc.
      */
      set procedure to :dUFLP:PixCalendar22.cc additive
      form.Cal = new PixCalendar22( form )
      form.Cal.top := 0
      form.Cal.left := 0
      form.Cal.Language := form.Parent.Language
   return

   function form_onOpen
      // these have to be set after the form has opened
      set procedure to :dUFLP:WinAttr32.CC additive
      set procedure to :dUFLP:CENTER32   additive

      // deal with Window attributes here:
      form.oWinAttr = new WinAttr32( form )
      form.oWinAttr.set( "caption", false ) // titlebar
      form.oWinAttr.MinButton(false)
      form.oWinAttr.MaxButton(false)
      form.oWinAttr.DlgFrame(false)
      form.oWinAttr.Border(true) // single-line border around window
      form.oWinAttr.ThickFrame(false)
      form.oWinAttr.SysMenu(false)

      // Set the date for the calendar:
      if not empty( this.parent.DateEntryfield.value )
         form.Cal.SetDate( form.parent.DateEntryfield.value )
      endif

      // Set Language specific text for pushbutton:
      do case
         case this.Cal.Language == "English"
              form.OKButton.text := "Select Date"
         case this.Cal.Language == "Spanish"
              form.OKButton.text := "Seleccione La Fecha"
         case this.Cal.Language == "French"
              form.OKButton.text := "Choisissez une date"
         case this.Cal.Language == "German"
              form.OKButton.text := "Whlen Sie Datum Vor"
         case this.Cal.Language == "Italian"
              form.OKButton.text := "Selezioni La Data"
      endcase
   return

   function OKClicked
      // Return the current date value, and close the form
      private d
      d = form.Cal.GetDate()
      if type( "d" ) == "O" // Date object
         form.parent.DateEntryfield.value := DTtoD( d )
      else
         form.parent.DateEntryfield.value := d
      endif
      form.close()
   return

   function form_onClose
      // close the winattr32 object:
      form.oWinAttr.close()
      form.oWinAttr := null
      form.parent.dropped := false // form is closed ...
   return

   function form_onLostFocus
      // if the form loses focus, we need to 
      // close but don't assign a value ...
      form.close()
   return

endclass

Function Smallfonts
/*
   Programmer   : Romain Strieff [dBVIPS]
   Last Modified: 04/20/1999
   Rev. History.: 04/20/1999 -- Original
   Notes........: Returns True if Smallfonts are used
   .............: in Windows menus
   Written for..: Visual dBASE (32-bit versions)
   Usage .......: ? SmallFonts()
*/
   local dc,result
   extern CINT        GetDeviceCaps(cHandle, CINT) gdi32
   extern CINT        ReleaseDC(cHandle, cHandle) user32
   extern cHandle     GetDC(cHandle) user32
   #define LOGPIXELSX    88
   dc=getdc(0)
   result=GetDeviceCaps(dc,LOGPIXELSX)
   ReleaseDc(0,dc)
return result=96
// eof: SmallFonts()

