/* GradientPainter class
---------------------------------------------------------------------
Program : gradientPainter.cc
Programmer : Greg Brzozowski GM+ (gmplus@interia.pl)
Last Modified : 12/02/2002

Written for.....: Visual dBASE 7.x , dB2K 0.x

Rev. History : 03/09/2001 -- Original
                       06/01/2002 -- works in dB2k 0.x                                        

Description:

	GreadientPainter class paints only Paintbox base class container.

Methods:

      Paint( <handle device context>, <fills rectangle>, <update rectangle>)
          Painting Gradient Fills

Properties:

     style          	 Gradient style : 1 - vertical, 2 - horizontal, 3 - diagonal

     colors 			One or two colors gradient  - "<color1>/<color2>"

     steps         	Number of steps in the gradient - 2..255
     stepStart 	 Starting operation from following number step in the gradient - 1..steps
     stepEnd 		 Ending operation to following number step in the gradient - stepStart..steps

     balance     	  Balance between two colors gradient in percent 1..50..100
     luminance 	Luminance colors gradient 


Dependencies :

		WinAPI_Window.prg
    	WinAPI_GDI.prg
      WinAPI_GDI.h

      funStruct.prg additive
      funColor.prg additive

Instantiation : 

	   do WinAPI_Window.prg
    	do WinAPI_GDI.prg

      set procedure to funStruct.prg additive
      set procedure to funColor.prg additive
      set procedure to gradientPainter.cc additive

     obj.paintBox = New PaintBox(obj)   
     obj.paintBox.onOpen = class::paintBox_onOpen
     obj.paintBox.onPaint = class::paintBox_onPaint

      function paintBox_onOpen

      //Pointer to structure for paint information:
      	this.structPaint = space(28)             

      //Pointer to rectangle structure:
     	  	 this.rectPaint = space(8)

      //Fills a rectangle Paintbox container:
          GetClientRect( this.hwnd, this.rectPaint) 
      //or sets the coordinates of the specified rectangle:
          SetRect(this.rectPaint, <left>, <top>, <right>, <bottom>)

     //Creates GradientPainter class:
     		this.gradientPainter = New GradientPainter()
         this.gradientPainter.style = 1
         this.gradientPainter.colors = "<color1>/<color2>"

      return

     function paintBox_onPaint
        
     //Handle to a display device context:
     	   local hdc
         hdc = BeginPaint( this.hwnd, this.structPaint)     

		//Painting gradient:
    		this.gradientPainter.Paint( hdc, this.rectPaint, Substr( this.structPaint, 5, 8))     

    	return Endpaint( this.hwnd, this.structPaint)

--------------------------------------------------------------*/



    class GradientPainter
    ************************************
     #Include WinAPI_GDI.h

     this.className = 'GRADIENTPAINTER'
    
     this.style = 1 
     this.colors = null
     this.steps = 85 
     this.stepStart = 1 
     this.stepEnd = 0 
     this.balance = 50
     this.luminance = 50   

         
   Function Paint( dcDest, grRect, upRect)
   *****************************************************    
   Local lSize, lSteps, xStep, yStep, nStart, nEnd,;           
             nPos, nCol, nCount, hBrush, aRGB, aPts,;
             prcBlnc, pwBlnc, left, top, right, bottom

   left = GetLong( grRect, 1)
   top = GetLong( grRect, 2)
   right = GetLong( grRect, 3)
   bottom = GetLong( grRect, 4)
                       
   lSteps = iif( this.steps > 1 and this.steps <= 255, this.steps, 85)       
   nStart = this.stepStart -1      
   nEnd = this.stepEnd -1
 
   if this.style = 3  
         
      lSize = Max( right -left, bottom -top)  
      nCount = iif( lSteps > lSize, lSize, lSteps)    
      xStep = ( right -left) / ( nCount /2)
      yStep = ( bottom -top) / ( nCount /2)                   
      nCount -= 1     
     
      Local hPen, nDiag
      hPen = SelectObject( dcDest, CreatePen( 5, 1, 0))  
      nDiag = nCount /2       
             
      if not Empty( upRect) 
         cx = ( right -left) / ( bottom -top)   
         nStart = Int( ( GetLong( upRect, 1) +GetLong( upRect, 2) *cx) /xStep)         
         nEnd = Int( ( GetLong( upRect, 3) +GetLong( upRect, 4) *cx) /xStep)      
      endif
                         
   else
      
      lSize = iif( this.style = 1, bottom -top, right -left)   
      nCount = iif( lSteps > lSize, lSize, lSteps) 
      xStep = ( right -left) /nCount
      yStep = ( bottom -top) /nCount        
      nCount -= 1

       if not Empty( upRect) 
          if this.style = 1
             nStart = Int( GetLong( upRect, 2) /yStep)
             nEnd = Int( GetLong( upRect, 4) /yStep)
          else
             nStart = Int( GetLong( upRect, 1) /xStep)
             nEnd = Int( GetLong( upRect, 3) /xStep)
          endif
       endif

   endif

   nStart = iif( nStart > 0 and nStart < nCount, nStart, 0) 
   nEnd = iif( nEnd => nStart and nEnd < nCount, nEnd, nCount)
    
   prcBlnc = iif( this.balance > 0 and this.balance <= 100, this.balance, 50) 
   pwBlnc = iif( prcBlnc # 50, (( 1.265 ^Abs( prcBlnc -50)) *.01) +1, 0)   
    
   prcLum = iif( this.luminance > 0 and this.luminance <= 100, this.luminance, 50) 
      
   nCol = TranslateColor( this.colors, 1)    
   if prcLum # 50 and '/' $ this.colors
      nCol = LumColor( nCol, ( prcLum *2) -100)
   endif
  
   aRGB = New Array( 3, 2)
   aRGB[1,1] = Bitand( nCol, 0xFF)
   aRGB[2,1] = Int( Bitand( nCol, 0xFF00) / 0x100)
   aRGB[3,1] = Int( Bitand( nCol, 0xFF0000) / 0x10000)
   
   if '/' $ this.colors
      nCol = TranslateColor( this.colors, 2) 
      if prcLum # 50
         nCol = LumColor( nCol, ( prcLum *2) -100)
      endif
   else
      nCol = LumColor( nCol, prcLum -100)  
   endif

   aRGB[1,2] = Bitand( nCol, 0xFF) - aRGB[1,1]
   aRGB[2,2] = Int( Bitand( nCol, 0xFF00) / 0x100) - aRGB[2,1]  
   aRGB[3,2] = Int( Bitand( nCol, 0xFF0000) / 0x10000) - aRGB[3,1]   
       
   aPts = New Array(3)
   aPts[1] = '        '   
   aPts[2] = '        '   
   aPts[3] = '        '   
   

   for nCur = nStart to nEnd
            
      if prcBlnc = 50
         nPos = ( nCur /nCount)
      elseif prcBlnc > 50
         nPos = ( nCur /nCount) ^pwBlnc
      elseif prcBlnc < 50
         nPos = 1 -(( nCount -nCur) /nCount) ^pwBlnc
      endif
         
      nCol = Bitand( aRGB[1,1] +aRGB[1,2] *nPos, 0xFF) ;
                +Bitand( aRGB[2,1] +aRGB[2,2] *nPos, 0xFF) *0x100 ;
                +Bitand( aRGB[3,1] +aRGB[3,2] *nPos, 0xFF) *0x10000     
             
      if this.style = 3   
         hBrush = SelectObject( dcDest, CreateSolidBrush( nCol))
                
         if nCur < nDiag                            
            lSteps = nCur +1                        
            cx = left +xStep *lSteps
            cy = top +yStep *lSteps
            SetRect( aPts[1], cx -xStep, top, left, cy -yStep)   
            SetRect( aPts[2], left, cy, cx, top)          
            Polygon( dcDest, aPts[1] +aPts[2], 4)   
                                                                     
         elseif nCur > nDiag              
            lSteps = nCur -nDiag +.5 
            cx = left +xStep *lSteps
            cy = top +yStep *lSteps
            SetRect( aPts[1], cx -xStep, bottom, right, cy -yStep)
            SetRect( aPts[2], right, cy, cx, bottom)                
            Polygon( dcDest, aPts[1] +aPts[2], 4)  
                               
         elseif nCur = nDiag    
            SetRect( aPts[1], left, top +yStep *nCur, left, bottom)
            SetRect( aPts[2], left +xStep /2, bottom, right, top +yStep /2)    
            SetRect( aPts[3], right, top, left +xStep *nCur, top)          
            Polygon( dcDest, aPts[1] +aPts[2] +aPts[3], 6)  
                      
         endif

         DeleteObject( SelectObject( dcDest, hBrush)) 

      else         
         hBrush = CreateSolidBrush( nCol)   
         lSteps = nCur +1       
             
         if this.style = 1        
            cy = top + yStep *lSteps
            SetRect( aPts[1], left, cy -yStep, right, cy)        
                       
         else
            cx = left + xStep *lSteps
            SetRect( aPts[1], cx -xStep, top, cx, bottom) 
                            
         endif  
         
         FillRect( dcDest, aPts[1], hBrush)           
         DeleteObject( hBrush)         
      endif    
     
   endfor

   if this.style = 3
      DeleteObject( SelectObject( dcDest, hPen))   
   endif

   return


 endclass

