/*
   Class: ArraySpinBox
   Author:  Daniel Howard sproket@total.net
            Gary White    garyw@qni.com (initial release)
   History: 
      1.0   March, 1 1998 
            Initial release

      2.0   December 12, 1998
            I completely rewrote this class so if you run into any
            problems don't yell at Gary!

   Properties:

      DataSource- Contains the array the user will see.
                  eg. 
                  this.DataSource = {"one","two","three"}

      Loop      - Boolean indicating how this will spin. If true then
                  selections will loop back to the first or last
                  element in the datasource. Otherwise the selection
                  will hold. Defaults to True.
   Notes:   
      See TreeView.Wfm for an example of it's use.

*----------------------------------------------------------------------
This software is donated to the public domain. You may modify 
this code in anyway you wish however the author reserves the 
right to whatever official version is published. If you have 
further ideas/bug reports/additions then contact the author and 
your name will be added to the contributors if your idea is 
implemented.

The author accepts no responsibility for the use or consequences 
of the use of this software.
*/
class ArraySpinbox(parentObj) of dCONTAINER(parentObj) custom
protect SpinBox_OnChange, ;
        Setter_CanChange, ;
        Setter_BeforeGetValue

   with (this)
      left = 1.8571
      top = 1.0909
      width = 20.5714
      height = 1.3636
      metric = 0	// Chars
   endwith


   this.SPINBOX = new AS_SPINBOX(this)
   with (this.SPINBOX)
      height = 1
      left = 12.8571
      top = 0.0455
      width = 6.5714
      value = 1
      rangeMin = 0
      rangeMax = 100  // Increase this for more than 100 elements
   endwith


   this.SPINFIELD = new AS_ENTRYFIELD(this)
   with (this.SPINFIELD)
      height = 1
      left = 0.1429
      top = 0.0455
      width = 16.4286
   endwith

   // Custom properties
   this.Loop = true
   this.DataSource = {"One","Two","Three","Four","Five"}

   this.Setter.OnChange = Class::Setter_OnChange
   this.Setter.CanChange = Class::Setter_CanChange
   this.Setter.BeforeGetValue = Class::Setter_BeforeGetValue
   this.Setter.Value := 1

   this.SpinBox.OnChange = Class::SpinBox_OnChange

   Function SpinBox_OnChange
      this.Parent.Setter.Value := this.Value
   return

   Function Setter_CanChange
   para nValue
      if type("nValue") # "N"
         msgbox("Invalid assignment!","Array Spinbox")
         return false
      endif

      local nMax
      nMax = this.Owner.DataSource.Size
            
      if this.Owner.Loop  // If we're looping
         if nValue > nMax
            nValue := 1
         elseif nValue < 1
            nValue := nMax
         endif
      else
         if nValue > nMax
            nValue := nMax
         elseif nValue < 1
            nValue := 1
         endif
      endif         
      this.CurSel = nValue
   return true

   Function Setter_OnChange
      this.Owner.SpinField.Value := this.Owner.DataSource[this.CurSel]
      this.Owner.SpinBox.Value := this.CurSel
   return

   Function Setter_BeforeGetValue
   return ( this.Owner.DataSource[this.CurSel] )
endclass

Class AS_SpinBox(oParent) of dSpinBox(oParent)
protect onChange,spinonly,step,datalink
   this.spinonly = true
   this.Step = 1
endclass

Class AS_Entryfield(oParent) of dEntryField(oParent)
protect onChange,when, mousePointer,datalink
   this.When = {; Return False }
   this.mousePointer = 1   // Arrow
endclass
