/*
    LegacyTable.cc
    This custom control was written by Romain Strieff, after he
    posted some code on the dBASE, Inc. newsgroups showing that
    you can create level 3 tables programmatically in Visual
    dBASE 7.x.

    This was modified by Gary White --
     * The 'version 5' code was removed, as it seemed redundant, 
       so this class will not work in VdBASE 5.x ...
     * Added a minimal amount of error checking
     * PROTECTed the CreateTable method
     * Changed from low-level file commands to using the
       file object ...
     * If safety is ON, it will prompt before over-writing,
       if safety is OFF, it will over-write without prompting ...

    This is a modified set of code, turned into a custom control,
    which allows you, the developer, to create a table of either
    level 3 or level 4 types, programmatically.

    WARNING: If you use fieldtypes that are level 5 or level 7,
    the BDE will convert the table to that type. This means,
    among other things, do not give fieldnames that are
    larger than 10 characters, and no spaces.

    Usage:

       set procedure to legacytable.cc additive
       lt=new legacyTable()
       lt.level=3                          // level three table
       lt.tablename="Mytable3.dbf"         // name of table
       lt.fields.add([Lastname char(10)])  // use local SQL format
       lt.fields.add([Firstname char(10)]) // for each field
       lt.save()                           // Creates the table
       lt.release()                        // release the object

     Properties:
       level         = numeric property,table level -- must be 3 or 4
       tableName     = character property, name of table
       fields        = array of field names and types

     Methods:
       save()        = saves the new table
       release()     = releases object
       createTable() = for internal use only

*/

class LegacyTable
 protect CreateTable
   //use this property to create level 3 and 4 tables
   this.level=3
   this.tablename=""
   this.fields=new array()

   function save
    local bSuccess
      bSuccess = true
      if this.createtable()
         for n=1 to this.fields.size
            cCommand=[alter table "]+this.tablename+;
                      [" add ]+this.fields[n]
            try
              &cCommand.
            catch(exception e)
              msgbox(e.message,'Error',16)
               bSuccess := false
               exit
            endtry
         next
         cTablename=this.tablename
         if bSuccess
          alter table "&cTablename." drop dummy
         else
          drop table "&cTablename."
         endif
      endif
   return bSuccess


   Function CreateTable
    local bRet
      bRet = true
      try
       if empty(this.tableName)
          throw new ltException('Table name is missing')
         endif
         if this.fields.size < 1
          throw new ltException('Fields array is empty')
         endif
         if set('safety')=='ON' and file(this.tablename)
            if msgbox('Overwrite existing table?','File exists',4+32)#6
             bRet := false
            endif
         endif
         if bRet
            a=new array()
            do case
               case this.level=3
                    a.add('03630C150000000041000200000000')
                    a.add('000000000000000000000000000000')
                    a.add('000044554D4D590000000000004305')
                    a.add('002E02010000000100000000000000')
                    a.add('000000000D1A')
               case this.level=4
                    a.add('03630C150000000041000200000000')
                    a.add('000000000000000000000000000058')
                    a.add('000044554D4D590000000000004300')
                    a.add('000000010000000000000000000000')
                    a.add('000000000D1A')
            endcase
            if not '.dbf' $ lower(this.tablename)
             this.tablename += '.dbf'
            endif
            f=new file()
            f.create(this.TableName, 'rw')
            for na=1 to a.size
                cString=a[na]
                for n=1 to len(cString) step 2
                    f.write(chr(htoi(substr(cString,n,2))))
                next
            next
            f.close()
         endif
      catch (exception e)
       bRet := false
         msgbox(e.message,'Error', 16)
      endtry
   return bRet


   Function Release
      release object this
   return

endclass

// end of class: LegacyTable

class ltException(cMsg) of exception
 this.message = cMsg
endclass

