// Metric: 6 - Pixels only
/*
   KENCAL2.CC
   Author  : Ken Mayer
   Original: 3/1998
   Updated : 06/12/1998
             09/01/1998 // very very minor updates
             10/22/1998 // changed calls to functions
                        // to use dateex.cc  and
                        // holiday.cc ... see
                        // dependencies list below
             10/30/1998 // Added check for form's metric
                        // problem: metrics for custom
                        // pushbuttons on a container
                        // do not calculate properly,
                        // this was designed with "Chars"
                        // for metrics -- if the form is
                        // not set to that metric, then
                        // the display will be ... odd.
             March 26, 1999 -- David Peters modified to
                        make this internationally independant
                        (meaning it can be used wherever)
                        by using the date class in dBASE.
             May 1, 1999 -- updated to fix a bug caused
                        by the above ...
             June 8, 2000 -- Updated to make this more 
                        self-contained, to fix some 
                        problems in the Holiday routines
                        (not finding all of the holidays),
                        and added speedTips to the 
                        dates that are holidays ...
             August 17, 2000 -- Minor: just added the 
                        currentDate and beginDate properties
                        to the constructor for the class.
             January 29, 2003 -- Modified to use dateex
                        class new method AddMonths2(), which
                        moves the date a little differently.
             March 2004 -- added some minor changes
                        to deal with some international issues.
             July 2004  -- Copied from kenCal2.cc and
                        modified to have a metric of 6 - Pixels,
                        this will only display properly on
                        a form that has this metric set.

                        Modified to have a Language property
                        that is based on the languages found
                        in the beginning of :dUFLP:DateEx.cc,
                        this can be used to set the month,
                        and day-of-week text controls automatically
                        based on information stored in the
                        Windows API. The only items
                        not translated would be the speedtips
                        for holidays, and the text for the
                        "Today" button. 

   This is a (mostly) self-contained calendar class -- no 
   ActiveX controls. It uses pushbuttons for dates and so on. 
   Note the dependencies section below -- for some of this
   to function properly, some other methods of the 
   dUFLP are involved.

   This version adds a bit of functionality to previous
   versions of KenCal2.CC in that holidays are now 
   shown in the "sunday color", making them stand out. 
   Holidays can have a separate color, and holidays that
   fall on Sunday will have the holiday color. In order
   to help differentiate which holiday is which, a
   speedtip appears for holidays if you hold the mouse
   over them ...

   Everything here is done with the OOP abilities of
   Visual dBASE. 

   NOTE: This does not resize well, so instead I made it 
   a bit on the small side, so it could be used on a dialog
   form, possibly with other controls -- see DTUI*.* and
   CalForm.wfm, as well as TestCal.wfm (which shows
   how to use CalForm and obtain a date ...) in
   the dUFLP library.

   Usage:
      set procedure to :dUFLP:kencal2.cc additive

      // in the forms designer, drop the container object
      // onto a form.

      The date can be SET in the form by referring to:

         form.pKenCalendarContainer1.currentDate

         (If you do not do this, it will use the 
         current date)

      To GET the date you just refer to the same
      value: 
         form.pKenCalendarContainer1.currentDate

      Assuming a form with a date entryfield:

      set procedure to myForm.wfm addi
      fMyForm = new MyFormForm()
      // use whatever value you need to -- you 
      // could use this in a modal form, for example,
      // and set the value to that in an entryfield,
      // and when the user closes the modal form,
      // reset the returned value to the entryfield ...
      fMyForm.pKenCalendarContainer1.currentDate := {01/01/1999}
      // *** OPTIONAL LANGUAGE STUFF:
      fMyForm.pKenCalendarContainer1.language := "German (Standard)"
      fMyForm.pKenCalendarContainer1.TodayPushbutton.text := "Heute"
      // *** End of Optional Language Stuff
      fMyForm.readModal()
      // when done, before releasing the modal form:
      form.entryfield1.value = ;
               fMyForm.pKenCalendarContainer1.currentDate
      // (where 'form.entryfield1' refers to the CALLING
      // form, and the entryfield that contains the date ...)

   NOTE: This stores a property to the container called beginDate
   which can be used if you include a "cancel" button ... -- you
   can reset the calendar control's currentDate property:

      form.pKenCalendarContainer1.currentDate := ;
         form.pKenCalendarContainer1.beginDate

   Example (of the original kenCal2.cc) can be found in 
   testcal.wfm and a form that uses the (original kenCal2.cc)
   control directly is CalForm.wfm

   COLOR PROPERTIES:
      The following custom color properties can be used
      to change the appearance of the calendar. These should
      be set probably in the form's open or onOpen event.

      form.pKenCalendarContainer1.NORMALCOLOR = "somecolor"
          This can be used to set the color of the non-selected
          days. If you leave off a background color, it should
          use the default "BTNFACE" (grey).
      form.pKenCalendarContainer1.SELECTEDCOLOR = "somecolor"
          This is used to set the selected day (current
          date) to a specific color so that it stands out.
          The default is red with a white background 
          ("red/white")
      form.pKenCalendarContainer1.SUNDAYCOLOR = "somecolor"
          As above, this is used to display the first column
          in a specific color. The default is red on
          the standard button background ("RED/BTNFACE")
          NOTE: In this version, holidays that
          are defined in the "LoadHolidays" method of the
          container will appear in the "SundayColor"
          unless they are also the "selected" day ...
      form.pKenCalendarContainer1.HOLIDAYCOLOR = "somecolor"
          As above, used specifically for Holidays, will
          override a sundaycolor setting ...

   LANGUAGE PROPERTIES:
      fMyForm.pKenCalendarContainer1.language := "German (Standard)"
          Note: this sets the language used based on 
                values found in the associative array in the
                beginning of DateEx.cc. The default is
                "English (United States)".
      fMyForm.pKenCalendarContainer1.TodayPushbutton.text := "Heute"
          This is important, as there is no good way for 
          me to find the correct text for this pushbutton.
          You could go further and change the text of
          the speedTips for the NextMonth, PreviousMonth,
          NextYear, PreviousYear pushbuttons as well.


   ** Note that this cc file uses Source Aliasing, specifically the
   ** dUFLP source Alias. If you do not know how to set up
   ** a Source Alias, see the instructions in WHATS.NEW at
   ** the top of the file, or in README.TXT.
   ** Any files referenced by :dUFLP:filename  must be
   ** included in an executable if you are building 
   ** one ... make sure that your project includes these
   ** files.
   **
   ** This particular program uses:
   **           dateex.cc
   **           holiday.cc
   **           time.cc
   **           hebrew.cc
   **           duflp.h <== not referred to by source alias
   **                       compiled into the object ...

*/

/*
   -------------------------------------------------------
   The container class here is what does all the work ...
   There are some custom button/text control definitions 
   after the container definition.
   -------------------------------------------------------
*/
class pKenCalendarContainer(parentObj) of CONTAINER(parentObj) custom
   set procedure to :dUFLP:dateex.cc additive
   set procedure to :dUFLP:holiday.cc additive

   with (this)
      width        := 132
      height       := 197
      metric       := 6	// Pixels
      transparent  := true
      onOpen       := class::CONTAINEROPEN
      onDesignOpen := class::CheckMetric
   endwith
   /*
      Color definitions are custom properties of the container
      used to define the buttons:
        NormalColor is the normal color used if not selected
        SelectedColor is what it sounds like, so that the
           currently selected date stands out
        SundayColor is used to show the first column
           (Sunday) ... a lot of calendars do this, so 
           I figured I would, too ...
        HolidayColor is used to show holidays ...

           Note that holidays that are in the array defined
           by "LoadHolidays" (a method of the container class)
           will appear in this color as well, unless they 
           are the "selected" item ...
   */
   this.NORMALCOLOR   = "BLACK/BTNFACE"
   this.SELECTEDCOLOR = "RED/WHITE"
   this.SUNDAYCOLOR   = "MAROON/BTNFACE"
   this.HOLIDAYCOLOR  = "WHITE/NAVY"
   this.Row1          =  62
   this.Row2          =  80
   this.Row3          =  98
   this.Row4          = 116
   this.Row5          = 134
   this.Row6          = 152
   this.Col1          =   7
   this.Col2          =  24
   this.Col3          =  40
   this.Col4          =  57
   this.Col5          =  73
   this.Col6          =  89
   this.Col7          = 105

   // instance of dateex class and holiday class:
   this.DateEx        = new DateEx()
   this.Holiday       = new Holiday()
   // used to set speedtips ...
   this.cSpeedTip     = ""
   // currentDate and beginDate:
   this.currentDate   = {}
   this.beginDate     = {}
   // Language -- hopefully for use with
   // the API calls in DateEx.cc, so we can
   // get the correct info for the month
   // and day of the week:
   this.Language      = "English (United States)"

   this.PREVYEARPUSHBUTTON = new PUSHBUTTON(this)
   with (this.PREVYEARPUSHBUTTON)
      onClick  := CLASS::PREVYEARPUSHBUTTON_OnClick
      upBitmap := "RESOURCE #852"
      text     := ""
      height   := 18
      width    := 16
      left     := 19
      top      := 3
      speedBar := true
      speedTip := "Previous Year"
   endwith

   this.TEXTYEAR = new TEXT(this)
   with (this.TEXTYEAR)
      fontSize        := 8
      text            := "9999"
      alignHorizontal := 1 // center
      alignVertical   := 1 // middle
      height          := 18
      left            := 37
      top             := 3
      width           := 56
      border          := true
      borderStyle     := 1
   endwith

   this.NEXTYEARPUSHBUTTON = new PUSHBUTTON(this)
   with (this.NEXTYEARPUSHBUTTON)
      onClick  := CLASS::NEXTYEARPUSHBUTTON_OnClick
      upBitmap := "RESOURCE #855"
      text     := ""
      height   := 18
      width    := 16
      left     := 95
      top      := 3
      speedBar := true
      speedTip := "Next Year"
   endwith

   this.PREVMONTHPUSHBUTTON = new PUSHBUTTON(this)
   with (this.PREVMONTHPUSHBUTTON)
      onClick  := CLASS::PREVMONTHPUSHBUTTON_OnClick
      upBitmap := "RESOURCE #852"
      text     := ""
      height   := 18
      width    := 16
      left     := 19
      top      := 21
      speedBar := true
      speedTip := "Previous Month"
   endwith

   this.TEXTMONTH = new TEXT(this)
   with (this.TEXTMONTH)
      height          := 18
      fontSize        := 8
      text            := "Month"
      alignHorizontal := 1 // center
      alignVertical   := 1 // middle
      left            := 37
      top             := 21
      width           := 56
      border          := true
      borderStyle     := 1
   endwith

   this.NEXTMONTHPUSHBUTTON = new PUSHBUTTON(this)
   with (this.NEXTMONTHPUSHBUTTON)
      onClick  := CLASS::NEXTMONTHPUSHBUTTON_OnClick
      height   := 18
      width    := 16
      left     := 95
      top      := 21
      upBitmap := "RESOURCE #855"
      text     := ""
      speedBar := true
      speedTip := "Next Month"
   endwith

   this.TITLESUNDAY = new TitleText(this)
   with (this.TITLESUNDAY)
      left := 7
      top  := 44
   endwith

   this.TITLEMONDAY = new TitleText(this)
   with (this.TITLEMONDAY)
      left := 24
      top  := 44
      text := "M"
   endwith


   this.TITLETUESDAY = new TitleText(this)
   with (this.TITLETUESDAY)
      left := 40
      top  := 44
      text := "T"
   endwith


   this.TITLEWEDNESDAY = new TitleText(this)
   with (this.TITLEWEDNESDAY)
      left := 57
      top  := 44
      text := "W"
   endwith


   this.TITLETHURSDAY = new TitleText(this)
   with (this.TITLETHURSDAY)
      left := 73
      top  := 44
      text := "T"
   endwith


   this.TITLEFRIDAY = new TitleText(this)
   with (this.TITLEFRIDAY)
      left := 89
      top  := 44
      text := "F"
   endwith


   this.TITLESATURDAY = new TitleText(this)
   with (this.TITLESATURDAY)
      left := 106
      top  := 44
   endwith

   this.ROW1COL1 = new DAYPUSHBUTTON(this)
   with (this.ROW1COL1)
      left = this.Col1
      top = this.Row1
   endwith

   this.ROW1COL2 = new DAYPUSHBUTTON(this)
   with (this.ROW1COL2)
      left = this.Col2
      top = this.Row1
   endwith

   this.ROW1COL3 = new DAYPUSHBUTTON(this)
   with (this.ROW1COL3)
      left = this.Col3
      top = this.Row1
   endwith

   this.ROW1COL4 = new DAYPUSHBUTTON(this)
   with (this.ROW1COL4)
      left = this.Col4
      top = this.Row1
   endwith

   this.ROW1COL5 = new DAYPUSHBUTTON(this)
   with (this.ROW1COL5)
      left = this.Col5
      top = this.Row1
   endwith

   this.ROW1COL6 = new DAYPUSHBUTTON(this)
   with (this.ROW1COL6)
      left = this.Col6
      top = this.Row1
   endwith

   this.ROW1COL7 = new DAYPUSHBUTTON(this)
   with (this.ROW1COL7)
      left = this.Col7
      top = this.Row1
   endwith

   this.ROW2COL1 = new DAYPUSHBUTTON(this)
   with (this.ROW2COL1)
      left = this.Col1
      top = this.Row2
   endwith

   this.ROW2COL2 = new DAYPUSHBUTTON(this)
   with (this.ROW2COL2)
      left = this.Col2
      top = this.Row2
   endwith

   this.ROW2COL3 = new DAYPUSHBUTTON(this)
   with (this.ROW2COL3)
      left = this.Col3
      top = this.Row2
   endwith

   this.ROW2COL4 = new DAYPUSHBUTTON(this)
   with (this.ROW2COL4)
      left = this.Col4
      top = this.Row2
   endwith

   this.ROW2COL5 = new DAYPUSHBUTTON(this)
   with (this.ROW2COL5)
      left = this.Col5
      top = this.Row2
   endwith

   this.ROW2COL6 = new DAYPUSHBUTTON(this)
   with (this.ROW2COL6)
      left = this.Col6
      top = this.Row2
   endwith

   this.ROW2COL7 = new DAYPUSHBUTTON(this)
   with (this.ROW2COL7)
      left = this.Col7
      top = this.Row2
   endwith

   this.ROW3COL1 = new DAYPUSHBUTTON(this)
   with (this.ROW3COL1)
      left = this.Col1
      top = this.Row3
   endwith

   this.ROW3COL2 = new DAYPUSHBUTTON(this)
   with (this.ROW3COL2)
      left = this.Col2
      top = this.Row3
   endwith

   this.ROW3COL3 = new DAYPUSHBUTTON(this)
   with (this.ROW3COL3)
      left = this.Col3
      top = this.Row3
   endwith

   this.ROW3COL4 = new DAYPUSHBUTTON(this)
   with (this.ROW3COL4)
      left = this.Col4
      top = this.Row3
   endwith

   this.ROW3COL5 = new DAYPUSHBUTTON(this)
   with (this.ROW3COL5)
      left = this.Col5
      top = this.Row3
   endwith

   this.ROW3COL6 = new DAYPUSHBUTTON(this)
   with (this.ROW3COL6)
      left = this.Col6
      top = this.Row3
   endwith

   this.ROW3COL7 = new DAYPUSHBUTTON(this)
   with (this.ROW3COL7)
      left = this.Col7
      top = this.Row3
   endwith

   this.ROW4COL1 = new DAYPUSHBUTTON(this)
   with (this.ROW4COL1)
      left = this.Col1
      top = this.Row4
   endwith

   this.ROW4COL2 = new DAYPUSHBUTTON(this)
   with (this.ROW4COL2)
      left = this.Col2
      top = this.Row4
   endwith

   this.ROW4COL3 = new DAYPUSHBUTTON(this)
   with (this.ROW4COL3)
      left = this.Col3
      top = this.Row4
   endwith

   this.ROW4COL4 = new DAYPUSHBUTTON(this)
   with (this.ROW4COL4)
      left = this.Col4
      top = this.Row4
   endwith

   this.ROW4COL5 = new DAYPUSHBUTTON(this)
   with (this.ROW4COL5)
      left = this.Col5
      top = this.Row4
   endwith

   this.ROW4COL6 = new DAYPUSHBUTTON(this)
   with (this.ROW4COL6)
      left = this.Col6
      top = this.Row4
   endwith

   this.ROW4COL7 = new DAYPUSHBUTTON(this)
   with (this.ROW4COL7)
      left = this.Col7
      top = this.Row4
   endwith

   this.ROW5COL1 = new DAYPUSHBUTTON(this)
   with (this.ROW5COL1)
      left = this.Col1
      top = this.Row5
   endwith

   this.ROW5COL2 = new DAYPUSHBUTTON(this)
   with (this.ROW5COL2)
      left = this.Col2
      top = this.Row5
   endwith

   this.ROW5COL3 = new DAYPUSHBUTTON(this)
   with (this.ROW5COL3)
      left = this.Col3
      top = this.Row5
   endwith

   this.ROW5COL4 = new DAYPUSHBUTTON(this)
   with (this.ROW5COL4)
      left = this.Col4
      top = this.Row5
   endwith

   this.ROW5COL5 = new DAYPUSHBUTTON(this)
   with (this.ROW5COL5)
      left = this.Col5
      top = this.Row5
   endwith

   this.ROW5COL6 = new DAYPUSHBUTTON(this)
   with (this.ROW5COL6)
      left = this.Col6
      top = this.Row5
   endwith

   this.ROW5COL7 = new DAYPUSHBUTTON(this)
   with (this.ROW5COL7)
      left = this.Col7
      top = this.Row5
   endwith

   this.ROW6COL1 = new DAYPUSHBUTTON(this)
   with (this.ROW6COL1)
      left = this.Col1
      top = this.Row6
   endwith

   this.ROW6COL2 = new DAYPUSHBUTTON(this)
   with (this.ROW6COL2)
      left = this.Col2
      top = this.Row6
   endwith

   this.ROW6COL3 = new DAYPUSHBUTTON(this)
   with (this.ROW6COL3)
      left = this.Col3
      top = this.Row6
   endwith

   this.ROW6COL4 = new DAYPUSHBUTTON(this)
   with (this.ROW6COL4)
      left = this.Col4
      top = this.Row6
   endwith

   this.ROW6COL5 = new DAYPUSHBUTTON(this)
   with (this.ROW6COL5)
      left = this.Col5
      top = this.Row6
   endwith

   this.ROW6COL6 = new DAYPUSHBUTTON(this)
   with (this.ROW6COL6)
      left = this.Col6
      top = this.Row6
   endwith

   this.ROW6COL7 = new DAYPUSHBUTTON(this)
   with (this.ROW6COL7)
      left = this.Col7
      top = this.Row6
   endwith

   this.TODAYPUSHBUTTON = new PUSHBUTTON(this)
   with (this.TODAYPUSHBUTTON)
      onClick = CLASS::TODAYPUSHBUTTON_OnClick
      height = 18
      left = 38
      top = 172
      width = 54
      text = ">> Today"
      upBitmap = ""
      fontSize = 8
      speedTip = "Go to Today's Date"
   endwith

   function CheckMetric
      // called from container's onDesignOpen:
      if form.metric # 6
         msgbox( "Form's metric property is not '6 - Pixels', "+;
                 "this will not display properly.", ;
                 "Bad Form Metric!", 16 )
         return
      endif

   function containerOpen
      // default date if needed:
      if type( 'this.currentDate' ) # "D" or ; 
         empty( this.currentDate )
         this.currentDate = date()
      endif

      // Set color of "sunday" title button
      this.titleSunday.colorNormal := this.SUNDAYCOLOR

      // store starting date value, in
      // case there's a cancel option,
      // the user can return to the 'beginDate':
      this.beginDate = this.currentDate

      #if __version__ > 2.5
         this.NextMonthPushbutton.systemTheme := false
         this.PrevMonthPushbutton.systemTheme := false
         this.NextYearPushbutton.systemTheme  := false
         this.PrevYearPushbutton.systemTheme  := false
      #endif

      // Update the text for the day of week headings:
      class::DayHeadings()

      // newMonth routine
      class::NewMonth()

   return

   function DayHeadings
         // deal with language stuff for the
      // day of the week:
      local cDay, dTemp

      // Set the language for the DateEx object:
      this.dateEx.Language := this.Language

      // we need a day known to be a Sunday:
      dTemp = new date( 2004, 1, 1 ) // February 1, 2004
                                     // remember date object
                                     // is zero-based, hence
                                     // the second month of
                                     // the year is 1
      // Get Sunday
      cDay = this.dateEx.intlcdow( dTemp )
      // Assign first letter ...:
      this.TitleSunday.text := left( cDay, 1 )
      // Get Monday
      cDay = this.dateEx.intlcdow( dTemp+1 )
      // Assign first letter ...:
      this.TitleMonday.text := left( cDay, 1 )
      // Get Tuesday
      cDay = this.dateEx.intlcdow( dTemp+2 )
      // Assign first letter ...:
      this.TitleTuesday.text := left( cDay, 1 )
      // Get Wednesday
      cDay = this.dateEx.intlcdow( dTemp+3 )
      // Assign first letter ...:
      this.TitleWednesday.text := left( cDay, 1 )
      // Get Thursday
      cDay = this.dateEx.intlcdow( dTemp+4 )
      // Assign first letter ...:
      this.TitleThursday.text := left( cDay, 1 )
      // Get Friday
      cDay = this.dateEx.intlcdow( dTemp+5 )
      // Assign first letter ...:
      this.TitleFriday.text := left( cDay, 1 )
      // Get Saturday
      cDay = this.dateEx.intlcdow( dTemp+6 )
      // Assign first letter ...:
      this.TitleSaturday.text := left( cDay, 1 )
   return

   function newMonth
      // this function deals with setting values based
      // on changes to month/year -- determine where 
      // first day is, last day is, everything in 
      // between: set button texts, colors ...

      // First: need to determine what level we're at --
      // are we working from container, or control
      // on container (cRef needs to point to the container):
      if "CONTAINER" $ upper( this.className ) or;
         "CONTAINER" $ upper( this.baseClassName )
         // we're on the container
         oRef = this  
      else
         // we're on a pushbutton _on_ the container
         oRef = this.parent 
      endif

      // Make sure holiday list is current ...
      class::LoadHolidays()

      // now to get first day of month and last day of
      // month from 'currentDate'
      dDate  = oRef.currentDate
      dFirst = oRef.dateEx.fdom( dDate )
      dLast  = oRef.dateEx.ldom( dDate )
      nDay   = day( dDate ) // current day of month

      // character values for month and year:
      oRef.textMonth.text := oRef.DateEx.IntlcMon( dDate )
      oRef.textYear.text  := ltrim( str( year( dDate ) ) )

      // now the fun part ... assigning values, based
      // on where we start:
      nStart = dow( oRef.currentDate )

      // Clear out any outstanding speedTips
      for nRows = 1 to 6
          for nColumns = 1 to 7
              cObject = "oRef.Row1Col"+nColumns
              cSpeedTip = cObject+".speedTip"
              &cSpeedTip. := ""
          next
      next

      // We need to loop through everything, and set
      // visible properties as well as text if it's going
      // to display:
      nCurrent = 0
      for nRows = 1 to 6
          for nColumns = 1 to 7
              nCurrent++
              // start at day 1, but ...
              // for first row, loop through until we find
              // the first one (and make everything before
              // the first day of month invisible):
              if nCurrent == 1
                 do while nCurrent <= nColumns
                    if dow( dFirst ) == nColumns
                       // set the text:
                       cObject = "oRef.Row1Col"+nColumns
                       cText   = cObject+".text"
                       &cText. := "1"
                       // Set colors:
                       cColor = cObject+".colorNormal"
                       // if the day we're setting is the
                       // selected/current date:
                       dTestDate = new date(year(dDate), ;
                                   month(dDate)-1, nCurrent)
                       // Make it a real date
                       dTestDate = ctod( left( dTestDate.toLocaleString(), ;
                                   iif( set("CENTURY") == "ON", 10, 8 ) ) )
                       if nDay == nCurrent
                          &cColor. := oRef.SELECTEDCOLOR
                          oRef.currentButton = &cObject.
                          cSpeedTip = cObject+".speedTip"
                          if class::isHoliday( dtestDate )
                             &cSpeedTip. = oRef.cSpeedTip
                          endif                       
                       else
                          // is it a sunday or a holiday?
                          cSpeedTip = cObject+".speedTip"
                          if nColumns == 1 
                             &cColor. := oRef.SUNDAYCOLOR
                             if class::IsHoliday( dTestDate )
                                &cColor.    := oRef.HOLIDAYCOLOR
                                &cSpeedTip. := oRef.cSpeedTip
                             endif
                          elseif class::IsHoliday( dTestDate )
                             &cColor. := oRef.HOLIDAYCOLOR
                             &cSpeedTip. := oRef.cSpeedTip
                          else
                             // it's any other day of week ...
                             &cColor. := oRef.NORMALCOLOR
                          endif
                       endif // nDay == nCurrent

                       // set visible property
                       cVisible = cObject+".visible"
                       &cVisible. := true
                       // finish the row
                       for i = nColumns to 7
                           cObject = "oRef.Row1Col"+i
                           cText   = cObject+".text"
                           &cText. := "1"
                           // set visible property
                           cVisible = cObject+".visible"
                           &cVisible. := true
                       next
                       exit // out of this loop, we're done
                    else 
                       if nColumns > 7
                          nColumns = 7
                          exit
                       endif
                       cObject = "oRef.Row1Col"+nColumns
                       cVisible = cObject+".visible"
                       &cVisible. := false
                    endif // dow( dFirst ) = nColumns
                    nColumns++ // increment column counter
                    loop       // top of _this_ loop
                 enddo // while nCurrent <= nColumns
                 loop // back to columns for/next loop
              endif // nCurrent = 1

              // now to attempt to handle setting
              // text and colors ...
              cObject = "oRef.Row"+nRows+"Col"+nColumns
              cSpeedTip = cObject+".speedTip"
              &cSpeedTip := "" // clear out whatever's there
              // if we're inside the month's dates:
              if nCurrent > 1 and nCurrent <= day( dLast )
                 // set the text property:
                 cText = cObject+".text"
                 &cText. := ""+nCurrent
                 // set the button to visible:
                 cVisible = cObject+".visible"
                 &cVisible. := true
                 // deal with colors:
                 cColor = cObject+".colorNormal"
                 // if the day we're setting is the
                 // selected/current date:
                 dTestDate = new date(year(dDate), ;
                             month(dDate)-1, nCurrent)
                 // Make it a real date
                 dTestDate = ctod( left( dTestDate.toLocaleString(), ;
                             iif( set("CENTURY") == "ON", 10, 8 ) ) )
                 if nDay == nCurrent
                    &cColor. := oRef.SELECTEDCOLOR
                    oRef.currentButton = &cObject.
                    cSpeedTip = cObject+".speedTip"
                    if class::isHoliday( dtestDate )
                       &cSpeedTip. = oRef.cSpeedTip
                    endif                       
                 else
                    // is it a sunday or a holiday?
                    cSpeedTip = cObject+".speedTip"
                    if nColumns == 1 
                       &cColor. := oRef.SUNDAYCOLOR
                       if class::IsHoliday( dTestDate )
                          &cColor.    := oRef.HOLIDAYCOLOR
                          &cSpeedTip. := oRef.cSpeedTip
                       endif
                    elseif class::IsHoliday( dTestDate )
                       &cColor. := oRef.HOLIDAYCOLOR
                       &cSpeedTip. := oRef.cSpeedTip
                    else
                       // it's any other day of week ...
                       &cColor. := oRef.NORMALCOLOR
                    endif
                 endif // nDay == nCurrent
              else
                 // we want to make it invisible
                 cVisible = cObject+".visible"
                 &cVisible. := false
              endif // nCurrent > 1 and nCurrent <= day( dLast )
          next // nColumns
       next // nRows

   function TODAYPUSHBUTTON_onClick
      // set today's date
      this.parent.currentDate = date()
      class::NewMonth()
   return

   function NEXTYEARPUSHBUTTON_onClick
      // code to add year to date,
      // and call newMonth routine
      this.parent.currentDate = ;
           this.parent.dateEx.addYears( this.parent.currentDate, 1 )
      class::NewMonth()      
   return

   function NEXTMONTHPUSHBUTTON_onClick
      // code to add month to date,
      // and call newMonth routine
      this.parent.currentDate = ;
           this.parent.dateEx.addMonths2( this.parent.currentDate, 1 )
      // Update the calendar:
      class::NewMonth()
   return

   function PREVMONTHPUSHBUTTON_onClick
      // code to subtract month from date,
      // and call newMonth routine
      this.parent.currentDate = ;
           this.parent.dateEx.addMonths2( this.parent.currentDate, -1 )
      class::NewMonth()     
   return

   function PREVYEARPUSHBUTTON_onClick
      // code to subtract year from date,
      // and call newMonth routine
      this.parent.currentDate = ;
           this.parent.dateEx.addYears( this.parent.currentDate, -1 )
      class::NewMonth()
   return

   procedure LoadHolidays
      // Creates/re-creates array used to determine
      // a holiday ... NOTE: It adds an array
      // to the form ...
      local nYear, oRef, nHoliday, d
      local cMarkSet
      private cCmd, cDateSet

      // Have to determine if being called from container
      // or from a button call ... 
      if "CONTAINER" $ this.className
         nYear = year( this.currentDate )
         oRef = this
      else
         nYear = year( this.parent.currentDate )
         oRef = this.parent
      endif

      // Create array
      oRef.Holidays = new Array(1,2) // 1 row, 2 columns
      
      // Load holidays using holidaydate method below:
      // floating holidays (most of these are US Holidays --
      // *********************************************
      // you may wish to modify if not in the US ...):
      // *********************************************
      // President's Day
      i = 0
      i++
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "P" )
      oRef.Holidays[i,2] = "President's Day"
      // Daylight Savings time -- always a Sunday
      // Memorial Day
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "M" )
      oRef.Holidays[i,2] = "Memorial Day"
      // Labor Day
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "L" )
      oRef.Holidays[i,2] = "Labor Day"
      // Columbus Day
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "C" )
      oRef.Holidays[i,2] = "Columbus Day"
      // Standard Time -- always a Sunday
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "S" )
      oRef.Holidays[i,2] = "Standard Time"
      // Daylight Savings Time
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "D" )
      oRef.Holidays[i,2] = "Daylight Savings Time"
      // Election Day
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "E" )
      oRef.Holidays[i,2] = "Election Day"
      // Thanksgiving
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "T" )
      oRef.Holidays[i,2] = "Thanksgiving"
      // Advent -- always a Sunday
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "A" )
      oRef.Holidays[i,2] = "Advent"

      // Easter requires special code ...:
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.EasterDay( nYear )
      oRef.Holidays[i,2] = "Easter"

      // fixed dates:
      // deal with date format, so we don't
      // mess up international users ...
      cDateSet = set("DATE")
      cMarkSet = set("MARK")
      set date MDY

      // New Year's Day
      d = ctod( "01/01/"+nYear )
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "New Year's Day"
      // Lincoln's Birthday
      d = ctod( "02/12/"+nYear)
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "Lincoln's Birthday"
      // Valentine's Day
      d = ctod( "02/14/"+nYear )
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "Valentine's Day"
      // Washington's Birthday
      d = ctod( "02/22/"+nYear)
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "Washington's Birthday"
      // St. Patrick's Day
      d = ctod( "03/17/"+nYear)
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "St. Patrick's Day"
      // April Fool's Day
      d = ctod( "04/01/"+nYear)
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "April Fool's Day"
      // Fourth of July
      d = ctod( "07/04/"+nYear )
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "Independance Day"
      // Halloween (All Hallows Eve)
      d = ctod( "10/31/"+nYear )
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "Halloween"
      // Christmas
      d = ctod( "12/25/"+nYear )
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "Christmas"
      // New Year's Eve
      d = ctod( "12/31/"+nYear )
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "New Year's Eve"

      // You can add more here if you wish ...
      // make sure you add a row to the array, 
      // and add the name of the holiday or memorable
      // date ... Basically, copy the code as shown
      // above, and make appropriate changes

      // reset date format to whatever it was
      // before we changed to American
      set date &cDateSet.
      cCmd = "set mark to '"+cMarkSet+"'"
      &cCmd.

      // one last thing:
      oRef.Holidays.sort(1) // sort on first column

   Function IsHoliday
      parameter dCheckIt
      // this routine reads the array "this.Holidays"
      // and checks the date to see if the date in
      // dCheck parameter is in the list, and returns
      // a logical value ... 
      if "CONTAINER" $ upper( this.className )
         oRef = this
      else
         oRef = this.parent
      endif
      lHoliday = false
      oRef.cSpeedTip = ""
      for i = 1 to oRef.Holidays.size/2
          if oRef.Holidays[ i, 1 ] == dCheckIt
             lHoliday := true
             oRef.cSpeedTip = oRef.Holidays[ i, 2 ]
          endif
      next
   return ( lHoliday )

endclass
// end of class pKenCalendarContainer

/* 
   ----------------------------------------------------------
   Pushbutton class definition:
   ----------------------------------------------------------
*/
class DayPushButton( oParent ) of PushButton( oParent ) //custom
   with (this)
      height   := 18
      width    := 16
      metric   := 6  // Pixels
      text     := "30"
      pageno   := 0
      visible  := false
      speedBar := true // so they don't keep focus
   endwith
   if __version__ > 2.5
      // for these toolbuttons, don't use the system theme ...
      this.systemTheme := false // for dBASE Plus 2.6 or later
   endif


   function onClick
      /*
         The tricky part of this little routine is
         resetting colors -- in order to do that,
         we need to store a reference to the 'current'
         button, so we can reset that, before we change
         the now current button's color ...
      */
      dCurrentDate = new date(year(this.parent.currentDate), ;
                     month(this.parent.currentDate)-1, ;
                     VAL(this.parent.currentButton.text))
      dCurrentDate = ctod( left( dCurrentDate.toLocaleString(), ;
                     iif( set("CENTURY") == "ON", 10, 8 ) ) )
      if type( "this.parent.currentbutton" ) # "U"
         // if it's in Column 1, it's a "sunday" ...
         if "COL1" $ this.parent.currentButton.Name 
            this.parent.currentButton.colorNormal := ;
                                  this.parent.SUNDAYCOLOR
         // set holiday color if needed:
         elseif this.parent.isHoliday( dCurrentDate )
               this.parent.currentButton.colorNormal := ;
                                  this.parent.HOLIDAYCOLOR
         // otherwise, we have a "normal" day ...
         else
            this.parent.currentButton.colorNormal := ;
                                    this.parent.NORMALCOLOR
         endif
      endif

      // Assign currentButton property of container
      this.parent.currentButton = this
      // Assign color ...
      this.colorNormal := this.parent.SELECTEDCOLOR

      // set current date to this one:
      this.parent.currentDate := ;
           new date( year(this.parent.currentDate), ;
                     month(this.parent.currentDate)-1, ;
                     VAL(this.text) )
      this.parent.currentDate := ;
                     ctod( left( this.parent.currentDate.toLocaleString(), ;
                     iif( set("CENTURY") == "ON", 10, 8 ) ) )

endclass

// custom class the "title" text so we have something nearly
// identical to the pushbuttons used for the days for the titles
class TitleText( oParent ) of Text( oParent ) custom
   with (this)
      height          := 16
      width           := 16
      metric          := 6  // Pixels
      text            := "S"
      pageno          := 0
      border          := true
      borderStyle     := 1 // Raised
      alignHorizontal := 1 // center
      alignVertical   := 1 // middle
   endwith

endclass

/*
   ================================================================
   End of file: KenCal2.CC
   ================================================================
*/


