/*

    TESTHTML.PRG
    Date...: 09/17/1998
    Authors: Ken Mayer and Gary White

    Program to test the HTMLCLAS.CC

    Attempts are made to test every feature of every tag covered
    in this class.

    The resulting HTML file is not "pretty" per se. This is an 
    attempt to test as many features in HTMLCLAS.CC as possible, 
    and therefore some of the output is a bit odd.

    You may want to examine the code to find how to use specific
    elements ... As noted in HTMLCLAS.CC, the custom class
    assumes you understand basic HTML, and so a full definition
    of what each tag does is not included. Specific attributes
    may be described but not in a lot of detail.

    Also, you may wish to note that in some cases Netscape
    and Internet Exploder do not interpret some of the tags
    in quite the same manner ... if you have both on a test
    machine, it may be a good idea to test any HTML you generate
    from VdBASE in both.

    I think it's all there -- a few attributes of some
    tags may not be being tested completely (most notably
    some of the NOWRAP type parameters in the table tags), 
    but that should be ok ...

   ** Note that this program uses Source Aliasing, specifically the
   ** dUFLP source Alias. If you do not know how to set up
   ** a Source Alias, see the instructions in WHATS.NEW at
   ** the top of the file, or in README.TXT.
   ** Any files referenced by :dUFLP:filename  must be
   ** included in an executable if you are building 
   ** one ... make sure that your project includes these
   ** files.
   **
   ** This particular program uses:
   **           htmlclas.cc

*/
clear
// set up the htmlclas object
set procedure to :dUFLP:htmlclas.cc additive
h=new htmlPage() // since no parm is passed, it will ask
                 // for the name of the file

if type("h") # "O"	// check to see if it was created
	return				// and exit if not
endif

// here we go:
h.HTMLStart()

// -----------------------------------------------------------------
// Header of document:
h.HeadStart()
h.Title( "A test of HTMLCLAS.CC" )
h.CommentStart()
h.write( "    This is a test of HTMLCLAS.CC" )
h.write( "    The purpose is to see if the tags" )
h.write( "    are all output correctly ..." )
h.write( "" )
h.write( "    HTMLCLAS.CC is by Ken Mayer and Gary White," )
h.write( "    and is a part of the dUFLP (freeware) library" ) 
h.write( "    for dBASE." )
h.write( "" )
h.write( "    Date generated: "+date() )
h.CommentEnd()
h.HeadEnd()

// -----------------------------------------------------------------
// Body of document -- this is, of course,
// where most of the testing will occur
h.CommentStart()
h.write( "   The body tag here doesn't include a background " )
h.write( "   image, as I don't have one handy at the moment." )
h.commentEnd()
h.BodyStart( "", "Navy","Aqua","Red","Lime","LightBlue" )

// -----------------------------------------------------------------
// Test the various tags ... text can be stuffed in
// by simply outputting it (write()), and of course a lot of testing
// needs to be done on the various formatting tags ...
h.write( "This is a test of a Visual dBASE HTML class, designed" )
h.write( "to create an HTML document that uses most of the HTML 3.2" )
h.write( "tags ..." )
h.paragraph()
h.write( "Background = 'NAVY'" )
h.break()
h.write( "Text = 'AQUA'" )
h.break()
h.write( "Hyperlinks = 'RED'" )
h.break()
h.write( "Visited links = 'GREEN' ('Lime')" )
h.break()
h.write( "Active links = 'LIGHTBLUE'" )
h.paragraph()

// -----------------------------------------------------------------
// Test formatting:
h.Rule()
h.Heading(1, "Formatting Tags (and Heading 1)" )
h.Address( "Address Tags" ) 
h.break()
h.AddressStart()
h.write( "Another test of Address" )
h.AddressEnd()
h.paragraph()

h.Bold( "BoldFace" )
h.break()
h.BoldStart() 
h.write( "More bold text" )
h.boldEnd() 
h.paragraph()

h.BlockStart()
h.write( "This text should be indented ... due to a blockquote tag. " )
h.write( "You can stuff lots of text in here, and it will all be " )
h.write( "indented up until we hit the end tag ... horizontal rules ")
h.write( "will be affected also: ")
h.rule()
h.BlockEnd()

h.write( "This text should be back to the left margin ..." )

h.Center( "This text should be centered" ) 
h.break()
h.CenterStart() 
h.write( "So should this" )
h.CenterEnd()
h.paragraph()

h.Heading( 2, "Heading size 2" ) 
h.Heading( 3, "Heading size 3" ) 
h.Heading( 4, "Heading size 4" ) 
h.Heading( 5, "Heading size 5" ) 
h.HeadingStart( 6 )
h.write( "Heading size 6" )
h.HeadingEnd( 6 ) 

h.Italic( "Italicized text" )
h.break()
h.ItalicStart()
h.write( "More italics" )
h.ItalicEnd()
h.Paragraph()

h.ListingStart()
h.write( "This is text inside a listing tag -- no formatting" )
h.write( "should <I>appear</I> -- all HTML tags should be " )
h.write( "<B>ignored</B> inside this area in the display." )
h.write( "Interesting, the text appears smaller than the" )
h.write( "default text -- IE? Don't know if this acts the" )
h.write( "same in Netscape ..." )
h.ListingEnd()
h.Paragraph()

h.NoBreak( "Non-breaking text -- a long line of text "+;
           "should not break -- it may run off the side "+;
           "of the screen, but it should not break ..." +;
           "here is more text -- we're just checking to see "+;
           "if the text will break at the right of the "+;
           "screen ..." )
h.Paragraph() 

h.Paragraph( "RIGHT" )
h.write( "This text should line up on the right" )
h.write( "side of the screen ..." )
h.ParagraphEnd()

h.Paragraph( "CENTER" )
h.write( "This text should be centered using paragraph" )
h.ParagraphEnd()

h.PreStart()
h.write( "This text is 'preformatted'. Unlike " )
h.write( "Listing, the PRE tag <I>should</I> recognize ")
h.write( "HTML tags, however ... note that the text is monospaced." )
h.PreEnd()
h.Paragraph()

h.write( "The following are 'rule' tests ..." )
h.break()
h.write( "Standard 'horizontal rule'" )
h.break()

h.rule()
h.write( "width=50%, align=LEFT" )
h.rule( "50%","","","LEFT" )
h.write( "size=5" )
h.rule( "", 5 )
h.write( "width=50%, size=5" )
h.rule( "50%", 5 )
h.write( "width=50%,NOSHADE,align=RIGHT" )
h.rule( "50%","","NOSHADE","RIGHT" )
h.write( "width=25%, align=CENTER" )
h.rule( "25%","","","CENTER" )
h.break()

h.Strikeout( "Text is struck-out" )
h.break()
h.StrikeoutStart() 
h.write( "More struckout text" )
h.StrikeoutEnd()
h.Paragraph()

h.ttStart()
h.write( "Typewriter (monospaced) text" )
h.break()
h.write( "This text requires breaks at the end of each" )
h.break()
h.write( "line or HTML will wrap like normal ..." )
h.ttEnd()
h.Paragraph()

h.Underline( "Underlined Text" )
h.break()
h.UnderlineStart() 
h.write( "More underlined text" )
h.UnderlineEnd()
h.Paragraph()

// -----------------------------------------------------------------
// End of "format" stuff, on to fonts:
h.rule()
h.write( "Basefont is about to be set to '4'" )
h.break()
h.basefont( 4 )
h.write( "The font displayed should be noticeably larger ..." )
h.basefont( 3 )
h.paragraph()

h.write( "Font size is about to be set to 4:" )
h.break()
h.fontStart( 4 )
h.write( "Test font ..." )
h.break()
h.fontStart( 4, "RED" )
h.write( "This should be size four, color red ..." )
h.break()
h.fontEnd()
h.paragraph()

// -----------------------------------------------------------------
// Tables:
h.rule()
h.write( "Table testing -- this will get pretty complex ..." )
h.paragraph()

h.write( "(First table) Table Parms: None (with caption on top)" )
h.Break()

h.CaptionStart( "TOP" ) 
h.write( "First Table Test -- No Parms" )
h.CaptionEnd()

h.TableStart()
   h.TableHeadStart()
	   h.write( "Column 1" )
   h.TableHeadEnd()
   h.TableHeadStart() 
	   h.write( "Column 2" )
   h.TableHeadEnd()
   h.TableHeadStart() 
 	  h.write( "Column 3" )
   h.TableHeadEnd()
   h.TableRowStart()
      h.TableCellStart()
	      h.write( "Row1,Col1" )
      h.TableCellEnd()
      h.TableCellStart()
   	   h.write( "Row1,Col2" )
      h.TableCellEnd()
      h.TableCellStart()
      	h.write( "Row1,Col3" )
      h.TableCellEnd()
   h.TableRowEnd()
   h.TableRowStart()
      h.TableCellStart()
	      h.write( "Row2,Col1" )
      h.TableCellEnd()
      h.TableCellStart()
   	   h.write( "Row2,Col2" )
      h.TableCellEnd()
      h.TableCellStart()
      	h.write( "Row2,Col3" )
      h.TableCellEnd()
   h.TableRowEnd()
h.TableEnd()
h.Paragraph()

h.write( "(Second table) Table Parms: Border = 2, CellSpacing = 5, CellPadding = 5, Width = 75% (with caption on top)" )
h.Break()

h.CaptionStart( "TOP" ) 
h.write( "Second Table Test -- Table Parms" )
h.CaptionEnd()
h.TableStart(2,5,5,"75%")
   h.TableHeadStart() 
	   h.write( "Column 1" )
   h.TableHeadEnd()
   h.TableHeadStart()
   	h.write( "Column 2" )
   h.TableHeadEnd()
   h.TableHeadStart()
	   h.write( "Column 3" )
   h.TableHeadEnd()
   h.TableRowStart()
		h.TableCell( "Row1,Col1" )
   	h.TableCell( "Row1,Col2" )
      h.TableCell( "Row1,Col3" )
   h.TableRowEnd()
   h.TableRowStart()
      h.TableCell( "Row2,Col1" )
      h.TableCell( "Row2,Col2" )
      h.TableCell( "Row2,Col3" )
   h.TableRowEnd()
h.TableEnd()
h.Paragraph()

h.write( "(Third table) Table Parms: Border = 1" )
h.Break()
h.write( "Table Heading Parms tested -- Test alignment" )
h.Break()

// IE 4 may ignore caption alignment? Source looks ok ... 
// check in Navigator ...

h.CaptionStart( "BOTTOM" )
h.write( "Third Table Test -- Table Heading Parms (with caption set to bottom)" )
h.CaptionEnd()
h.TableStart(1)
h.TableHeadStart("LEFT","TOP")
	h.write( "Column 1" )
h.TableHeadEnd()
h.TableHeadStart("CENTER","MIDDLE")
	h.write( "Column 2" )
h.TableHeadEnd()
h.TableHeadStart("RIGHT","BOTTOM")
	h.write( "Column 3" )
h.TableHeadEnd()

h.TableHeadStart()
h.write( "Dummy<BR>Header" )
h.TableHeadEnd()
h.TableRowStart()
	h.TableCell( "Row1,Col1" )
	h.TableCell( "Row1,Col2" )
	h.TableCell( "Row1,Col3" )
h.TableRowEnd()
h.TableRowStart()
	h.TableCell( "Row2,Col1" )
	h.TableCell( "Row2,Col2" )
	h.TableCell( "Row2,Col3" )
h.TableRowEnd()
h.TableEnd()
h.Paragraph()

h.write( "(Fourth table) Table Parms: Border = 1 (no caption)" )
h.Break()
h.write( "Table Heading Parms tested -- Test (horizontal) spanning" )
h.Break()

h.TableStart(1)
h.TableHeadStart("","",2) 
h.write( "Column 1" )
h.TableHeadEnd()
h.TableHeadStart()
h.write( "Column 3" )
h.TableHeadEnd()
h.TableRowStart()
h.TableCellStart()
h.write( "Row1,Col1" )
h.TableCellEnd()
h.TableCellStart()
h.write( "Row1,Col2" )
h.TableCellEnd()
h.TableCellStart()
h.write( "Row1,Col3" )
h.TableCellEnd()
h.TableRowEnd()
h.TableRowStart()
h.TableCellStart()
h.write( "Row2,Col1" )
h.TableCellEnd()
h.TableCellStart()
h.write( "Row2,Col2" )
h.TableCellEnd()
h.TableCellStart()
h.write( "Row2,Col3" )
h.TableCellEnd()
h.TableRowEnd()
h.TableEnd()
h.Paragraph()

h.write( "(Fifth table) Table Parms: Border = 1 (no caption)" )
h.Break()
h.write( "Table Heading Parms tested -- Test (vertical) spanning" )
h.Break()

h.TableStart(1)
h.TableHeadStart()
h.TableHeadEnd()
h.TableHeadStart()
h.write( "Column 1" )
h.TableHeadEnd()
h.TableHeadStart()
h.write( "Column 2" )
h.TableHeadEnd()
h.TableHeadStart() 
h.write( "Column 3" )
h.TableHeadEnd()
h.TableRowStart()
h.TableHeadStart("","","",2)
h.write( "Rows 1&2" )
h.TableHeadEnd()
h.TableCellStart()
h.write( "Row1,Col1" )
h.TableCellEnd()
h.TableCellStart()
h.write( "Row1,Col2" )
h.TableCellEnd()
h.TableCellStart()
h.write( "Row1,Col3" )
h.TableCellEnd()
h.TableRowEnd()
h.TableRowStart()
h.TableCellStart()
h.write( "Row2,Col1" )
h.TableCellEnd()
h.TableCellStart()
h.write( "Row2,Col2" )
h.TableCellEnd()
h.TableCellStart()
h.write( "Row2,Col3" )
h.TableCellEnd()
h.TableRowEnd()
h.TableEnd()
h.Paragraph()

h.write( "(Sixth table) Table Parms: Border = 1 (no caption)" )
h.Break()
h.write( "Table Row Parms tested -- alignment" )
h.Break()

h.TableStart(1)
h.TableHeadStart() 
h.write( "Column 1" )
h.TableHeadEnd() 
h.TableHeadStart() 
h.write( "Column 2" )
h.TableHeadEnd() 
h.TableHeadStart() 
h.write( "Column 3" )
h.TableHeadEnd() 
h.TableRowStart("LEFT","TOP") 
h.TableCellStart() 
h.write( "Row1,Col1" )
h.TableCellEnd() 
h.TableCellStart() 
h.write( "Row1, Col2" )
h.TableCellEnd() 
h.TableCellStart() 
h.write( "Row1, Column3" )
h.TableCellEnd() 
h.TableRowEnd() 
h.TableRowStart("CENTER","MIDDLE") 
h.TableCellStart() 
h.write( "Row2,Col1" )
h.TableCellEnd() 
h.TableCellStart() 
h.write( "Row2,Col2" )
h.TableCellEnd() 
h.TableCellStart() 
h.write( "Row2,Col3" )
h.TableCellEnd() 
h.TableRowEnd() 
h.TableRowStart("RIGHT","BOTTOM") 
h.TableCellStart() 
h.write( "Row2,<BR>Col1" )
h.TableCellEnd() 
h.TableCellStart() 
h.write( "Row2,Col2" )
h.TableCellEnd() 
h.TableCellStart() 
h.write( "Row2,Col3" )
h.TableCellEnd() 
h.TableRowEnd() 
h.TableEnd() 
h.Paragraph() 

h.write( "(Seventh table) Table Parms: Border = 1 (no caption)" )
h.Break()
h.write( "Table Cell (Data) Parms tested" )
h.Break()

h.TableStart(1)
h.TableHeadStart()
h.write( "Column 1" )
h.TableHeadEnd()
h.TableHeadStart()
h.write( "Column 2" )
h.TableHeadEnd()
h.TableHeadStart()
h.write( "Column 3" )
h.TableHeadEnd()
h.TableRowStart()
// Alignment
h.TableCellStart("LEFT","TOP")
h.write( "Row1,Col1" )
h.TableCellEnd()
h.TableCellStart("RIGHT","MIDDLE")
h.write( "Row1,<BR> Col2" )
h.TableCellEnd()
h.TableCellStart("CENTER","BOTTOM")
h.write( "Row1, Column3" )
h.TableCellEnd()
h.TableRowEnd()
h.TableRowStart()
// spanning
h.TableCellStart("","",2) 
h.write( "Row2,Col1" )
h.TableCellEnd()
h.TableCellStart("","","",2)
h.write( "Row2,Col3" )
h.TableCellEnd()
h.TableRowEnd()
// width
h.TableRowStart()
h.TableCellStart("","","","","50%") 
h.write( "Row2,<BR>Col1" )
h.TableCellEnd()
h.TableCellStart()
h.write( "Row2,Col2" )
h.TableCellEnd()
h.TableCellStart() 
h.write( "Row2,Col3" )
h.TableCellEnd()
h.TableRowEnd() 
h.TableEnd() 
h.Paragraph()

// -----------------------------------------------------------------
// Image:
h.rule()
h.write( "Image testing" )
h.break()
#ifdef __vdb__ < 2000 // Visual dBASE 7.x
   image = "file:///"+_DBWINHOME+"SAMPLES\FLEET\WAND.GIF"
#elseif __version__ < 2 // dB2K
   image = "file:///"+_DBWINHOME+"Images\PoweredBy\PowerdBydB2K.GIF"
#else // dBASE Plus
   image = "file:///"+_DBWINHOME+"Media\Images\Magglass.bmp"
#endif
h.write( "Just the image:" )
h.break()
h.Image( image )
h.break("LEFT")
h.write( "Image with alt='logo' and a bunch of alignment tests:" )
h.break()
h.Image( image, "Logo", "TOP" ) 
h.write( "Text for alignment test (TOP)" )
h.break("LEFT")
h.Image( image, "Logo", "MIDDLE" )
h.write( "Text for alignment test (MIDDLE)" )

h.break("LEFT")
h.Image( image, "Logo", "BOTTOM" )
h.write( "Text for alignment test (BOTTOM)" )
h.break("LEFT")
h.Image( image, "Logo", "LEFT" )
h.write( "Text for alignment test (LEFT)" )
h.break("LEFT")
h.Image( image, "Logo", "RIGHT" )
h.write( "Text for alignment test (RIGHT)" )
h.break("RIGHT")
h.Image( image, "Logo", "TEXTTOP" )
h.write( "Text for alignment test (TEXTTOP)" )
h.break("LEFT") 
h.Image( image, "Logo", "ABSMIDDLE" )
h.write( "Text for alignment test (ABSMIDDLE)" )
h.break("LEFT") 
h.Image( image, "Logo", "BASELINE" )
h.write( "Text for alignment test (BASELINE)" )
h.break("LEFT") 
h.Image( image, "Logo", "ABSBOTTOM" )
h.write( "Text for alignment test (ABSBOTTOM)" )
h.break("LEFT") 
h.Paragraph() 


h.write( "Image with alt='logo', align='top', hspace=5, vspace=5")
h.break()
h.Image( image, "Logo", "TOP", 10, 10 )
h.write( "Text for spacing test" )
h.break("LEFT")

h.write( "Image with alt='logo', align='top', width=20,height=20" )
h.break()
h.Image( image, "Logo", "TOP", "", "", 120, 120 )
h.write( "Text for spacing test" )
h.break("LEFT")

h.write( "Image with alt='logo', align='top', border=2 " )
h.Italic("(only appears with anchor/link)")
h.break()
h.LinkStart( "#END" )
h.Image( image, "Logo", "TOP", "", "", "", "", 2 )
h.write( "Link to end of document" )
h.LinkEnd()

// -----------------------------------------------------------------
// Anchors:
h.rule()
h.write( "Anchor testing" )
h.break()
h.LinkName( "BACK" ) 
h.write( "This is a link named 'BACK'" )
h.LinkEnd()
h.Break()
h.LinkStart( "#END" )
h.write( "End of this document" )
h.LinkEnd() 
h.Break()
h.MailToStart( "test@tester.test" ) 
h.write( "Test email" )
h.LinkEnd()
h.Paragraph()
h.write( "EMail Obscure Test: " )
h.Break()
h.EMailObscure( "Test@tester.test", "EMail Test" )
h.Paragraph()

// -----------------------------------------------------------------
// Lists:
h.rule()
h.write( "List tests" )
h.Break()

// Ordered Lists:
h.write( "Ordered List, no parameters")
h.OrderedListStart()
h.ListItem();h.write( "Item 1" )
h.ListItem();h.write( "Item 2" )
h.ListItem();h.write( "Item 3" )
h.ListItem();h.write( "Item 4" )
h.ListItem();h.write( "Item 5" )
h.ListItem();h.write( "Item 6" )
h.OrderedListEnd()

h.write( "Ordered List, Type='A', Start = 3")
h.OrderedListStart("A",3)
h.ListItem();h.write( "Item 1" )
h.ListItem();h.write( "Item 2" )
h.ListItem();h.write( "Item 3" )
h.ListItem();h.write( "Item 4" )
h.ListItem();h.write( "Item 5" )
h.ListItem();h.write( "Item 6" )
h.OrderedListEnd()

h.write( "Ordered List, Type='a'")
h.OrderedListStart("a")
h.ListItem();h.write( "Item 1" )
h.ListItem();h.write( "Item 2" )
h.ListItem();h.write( "Item 3" )
h.ListItem();h.write( "Item 4" )
h.ListItem();h.write( "Item 5" )
h.ListItem();h.write( "Item 6" )
h.OrderedListEnd()

h.write( "Ordered List, Type='I'")
h.OrderedListStart("I")
h.ListItem();h.write( "Item 1" )
h.ListItem();h.write( "Item 2" )
h.ListItem();h.write( "Item 3" )
h.ListItem();h.write( "Item 4" )
h.ListItem();h.write( "Item 5" )
h.ListItem();h.write( "Item 6" )
h.OrderedListEnd()

h.write( "Ordered List, Type='i'")
h.OrderedListStart("i")
h.ListItem();h.write( "Item 1" )
h.ListItem();h.write( "Item 2" )
h.ListItem();h.write( "Item 3" )
h.ListItem();h.write( "Item 4" )
h.ListItem();h.write( "Item 5" )
h.ListItem();h.write( "Item 6" )
h.OrderedListEnd()

h.write( "Ordered List, Type='1' (default, actually)")
h.OrderedListStart("1")
h.ListItem();h.write( "Item 1" )
h.ListItem();h.write( "Item 2" )
h.ListItem();h.write( "Item 3" )
h.ListItem();h.write( "Item 4" )
h.ListItem();h.write( "Item 5" )
h.ListItem();h.write( "Item 6" )
h.OrderedListEnd()

h.write( "Ordered List, List Item parameters tested")
h.OrderedListStart()
h.ListItem();h.write( "Item 1" )
h.ListItem("A");h.write( "Item 2" )
h.ListItem("a");h.write( "Item 3" )
h.ListItem("I");h.write( "Item 4" )
h.ListItem("i");h.write( "Item 5" )
h.ListItem("1");h.write( "Item 6" )
h.OrderedListEnd()

h.write( "Nested Ordered List")
h.OrderedListStart("I")
h.ListItem();h.write( "Item 1" )
  h.OrderedListStart("i")
  h.ListItem();h.write( "SubItem1")
  h.ListItem();h.write( "SubItem2")
  h.OrderedListEnd()
h.ListItem();h.write( "Item 2" )
  h.OrderedListStart("1") 
  h.ListItem();h.write( "SubItem1")
  h.ListItem();h.write( "SubItem2")
  h.OrderedListEnd()
h.ListItem();h.write( "Item 3" )
  h.OrderedListStart("A")
  h.ListItem();h.write( "SubItem1")
  h.ListItem();h.write( "SubItem2")
  h.OrderedListEnd()
h.ListItem();h.write( "Item 4" )
  h.OrderedListStart("a")
  h.ListItem();h.write( "SubItem1")
  h.ListItem();h.write( "SubItem2")
  h.OrderedListEnd()
h.ListItem();h.write( "Item 5" )
h.ListItem();h.write( "Item 6" )
h.OrderedListEnd()

h.write( "IE appears to ignore parameters for unordered lists ..." )
h.break()
h.write( "Unordered List, No parameters")
h.UnorderedListStart()
h.ListItem();h.write( "Item 1" )
h.ListItem();h.write( "Item 2" )
h.ListItem();h.write( "Item 3" )
h.ListItem();h.write( "Item 4" )
h.ListItem();h.write( "Item 5" )
h.ListItem();h.write( "Item 6" )
h.UnorderedListEnd()

h.write( "Unordered List, Type='DISC'")
h.UnorderedListStart('DISC')
h.ListItem();h.write( "Item 1" )
h.ListItem();h.write( "Item 2" )
h.ListItem();h.write( "Item 3" )
h.ListItem();h.write( "Item 4" )
h.ListItem();h.write( "Item 5" )
h.ListItem();h.write( "Item 6" )
h.UnorderedListEnd()

h.write( "Unordered List, Type='SQUARE'")
h.UnorderedListStart('SQUARE')
h.ListItem();h.write( "Item 1" )
h.ListItem();h.write( "Item 2" )
h.ListItem();h.write( "Item 3" )
h.ListItem();h.write( "Item 4" )
h.ListItem();h.write( "Item 5" )
h.ListItem();h.write( "Item 6" )
h.UnorderedListEnd()

h.write( "Unordered List, Type='CIRCLE' (default)")
h.UnorderedListStart('CIRCLE')
h.ListItem();h.write( "Item 1" )
h.ListItem();h.write( "Item 2" )
h.ListItem();h.write( "Item 3" )
h.ListItem();h.write( "Item 4" )
h.ListItem();h.write( "Item 5" )
h.ListItem();h.write( "Item 6" )
h.UnorderedListEnd()

h.write( "Unordered List, ListItem Parameters")
h.UnorderedListStart()
h.ListItem("CIRCLE") ;h.write( "Item 1" )
h.ListItem("SQUARE") ;h.write( "Item 2" )
h.ListItem("DISC")   ;h.write( "Item 3" )
h.ListItem("CIRCLE") ;h.write( "Item 4" )
h.ListItem("SQUARE") ;h.write( "Item 5" )
h.ListItem("DISC")   ;h.write( "Item 6" )
h.UnorderedListEnd()

h.write( "Nested Unordered List")
h.UnorderedListStart("SQUARE")
h.ListItem() ;h.write( "Item 1" )
  h.UnorderedListStart()
  h.ListItem();h.write( "SubItem1")
  h.ListItem();h.write( "SubItem2")
  h.UnorderedListEnd()
h.ListItem() ;h.write(  "Item 2" )
  h.UnorderedListStart("DISC")
  h.ListItem() ;h.write( "SubItem1")
  h.ListItem() ;h.write( "SubItem2")
  h.UnorderedListEnd()
h.ListItem() ;h.write( "Item 3" )
  h.UnorderedListStart("CIRCLE")
  h.ListItem() ;h.write("SubItem1")
  h.ListItem() ;h.write("SubItem2")
  h.UnorderedListEnd()
h.ListItem() ;h.write( "Item 4" )
  h.UnorderedListStart("SQUARE")
  h.ListItem() ;h.write( "SubItem1")
  h.ListItem() ;h.write( "SubItem2")
  h.UnorderedListEnd()
h.ListItem() ;h.write( "Item 5" )
h.ListItem() ;h.write( "Item 6" )
h.UnorderedListEnd()

// -----------------------------------------------------------------
// duh end
h.rule()
h.LinkName( "END" )
h.LinkEnd()
h.LinkStart( "#BACK" )
h.write( "Back to 'Anchors' section" )
h.LinkEnd()
h.break()

h.write( "That is the end of the test -- all the tags" )
h.write( "covered in HTMLCLAS.CC have now been tested ..." )

// end of body:
h.BodyEnd()

// close it:
h.closeFile()

// done!
msgbox( "File:"+chr(13)+h.FileName +chr(13)+ " has been created", "Done", 64 )


/*
    ---------------------------------------------------------------
    That's all folks ...
    ---------------------------------------------------------------
*/
