/*********************************************************************************

	File: transtext.cc
	Author: Marc Van den Berghen
	Last modified: 27-04-2007
	Version: 1.0  First beta
			   1.1  added invalidaterect to avoid repainting-problems.
            	  Thanks to Gary White, it works in all metrics 
            1.2  added textcolor  (thanks to Rich (Autotraker)
            	  changed the instantiation of the object so that 
                 it doesn't use macro substitution
                 added method settext, to change text on the fly

	Description: draws a text with a true transparent background

   Be sure to  insert the following line in your code

   Set Procedure to Transtext.cc additive

   In the form designer, drag a mvTranstext object from the
   Custom page of the component palette to the form (Container)
   It will appear as a textlabelobject in designmode and will be 
   a transparent backgound-text in run-mode 

   The method Settext will change the text on the fly

   this.text = "New text"
   this.settext( 0x80FF80 )  // the parameter is used to set the color of the text
   
   -----------------------------------------------------------
   Warranty:
   This code is freeware and there is no warranty. Use this code at 
   your own risk. 
   -----------------------------------------------------------

   Remark : with the colorado.cc one could automate the representation
   of the textcolor. However, the colorado class has a huge associate array
   to work with. This would increase the memory-need of the transparent 
   text control considerably, therefor i decided to follow Richs path
   and set the color manualy.
   The Settext-method takes a value that will set the color of the text

**/

#define LOGPIXELSY 		  90
#define DT_SINGLELINE     32
#define WM_NCPAINT        0x0085

class mvTransText(o) of textlabel(o)

	this.colorref = 0
   this.onopen=class::onopen

   function onOpen
      initialize()       // do the externs outside for better readability
  		this.visible=false // switch the textlabel off, it is used only for
      	    				 // design purpose 

      this.zw 			= new tpaintbox(this.parent)
		this.zw.base   = this
      this.zw.settext(this.colorref)
   return

   function settext(co)
   	this.zw.settext(co)
   return

endclass

function initialize()
      if type('DeleteObject') # 'FP'
         extern clogical DeleteObject(chandle) gdi32
      endif
      if type('SelectObject') # 'FP'
		   extern chandle SelectObject(chandle, chandle) gdi32
      endif
      if type('DrawText') # 'FP'
		   extern cint DrawText(chandle, cstring, cint, cptr, cuint) user32 ;
                   from "DrawTextA"
      endif
		if type('CreateFont') # 'FP'
		   extern chandle CreateFont(cint, cint, cint, cint, cint, culong, ;
	      culong, culong, culong, culong, culong, culong, culong, cstring) gdi32 ;
	      from 'CreateFontA'
		endif
      if type('SetBkMode') # 'FP'
         extern cint SetBkMode(chandle, cint) gdi32
      endif
	   if type('MulDiv') # 'FP'
	      extern cint  MulDiv(cint, cint, cint) kernel32
	   endif
	   if type('GetDeviceCaps') # 'FP'
	      extern cint GetDeviceCaps(chandle, cint) gdi32
	   endif
		if type('BeginPaint') # 'FP'
		   extern chandle BeginPaint(chandle, cptr) user32
		endif 
		if type('EndPaint') # 'FP'
		   extern cvoid EndPaint(chandle, cptr) USER32
		endif 
		if type('GetClientRect') # 'FP'
	      extern clogical GetClientRect(chandle, cptr) user32
		endif 
		if type('InvalidateRect') # 'FP'
		   extern clogical InvalidateRect(chandle, cptr, clogical) user32
      endif
      if type('SetParent') # 'FP'
         extern chandle SetParent(chandle,chandle) user32
      endif
      if type('RedrawWindow') # 'FP'
			extern clogical RedrawWindow(chandle, cptr, chandle, cuint) user32
		endif
      if type('SetTextColor') # 'FP'
         extern CULONG SetTextColor(CHANDLE,CULONG) GDI32
      endif

      extern clogical SendMessageA(chandle,cuint,clong,clong) user32
      extern chandle GetDC(chandle) user32
      extern cint ReleaseDC(chandle,chandle) user32
return

Class Tpaintbox(oparent) of paintbox(oparent)

	this.colornormal = ""
   this.borderstyle=3
   this.transparent=true
   this.colorref = 0

   this.base = Null
   this.hfont= 0
	this.onpaint = class::dotext
   this.onclose = class::cleanup

   function settext(colorref)

   	local hdc

      ** if a color is passed as parameter to this method
      ** it will be used, to draw the foreground color of the
      ** transparent textcontrol

      if not empty(colorref)
      	this.colorref=colorref
      endif

		this.left   = this.base.left
      this.top    = this.base.top            
     	this.height = this.base.height
      this.width  = this.base.width

      hdc = getdc(this.hwnd)
      if this.hfont<>0
	      deleteobject(this.hfont) // clear the font
   	endif

      this.lfHeight = - MulDiv(this.base.fontsize, GetDeviceCaps(hDC, LOGPIXELSY), 72)
      this.hfont = CreateFont(this.lfHeight, 0, 0, 0, ;
               iif(this.base.fontBold, 700,0), ;
               iif(this.base.fontItalic, 1, 0), ;
               iif(this.base.fontUnderline, 1, 0), ;
               iif(this.base.fontStrikeout, 1, 0), ;
               0, 0, 0, 2, 0, this.base.fontName)
      this.uformat=this.base.alignvertical+this.base.alignhorizontal+DT_SINGLELINE
		this.width ++
      this.width --
      releasedc(this.hwnd,hdc)
	return

   function dotext
	  local hdc,textrect	
     textrect=space(8)
     getclientrect(this.hwnd,textrect)
     invalidaterect(this.hwnd,textrect,false)
     paintstruct = space(32)   // structure for Beginpaint	
     hdc=BeginPaint(this.hwnd, paintstruct)
     SelectObject(hdc,this.hfont) 
     oldbkmode=setbkmode(hdc,1)           // make sure that the background is transparent
     SetTextColor(hDC,this.colorref)      // set the color
     ? this.colorref
     drawtext(hdc,this.base.text,len(this.base.text),textrect,this.uformat)
     setbkmode(hdc,oldbkmode)
     EndPaint(this.hwnd, paintstruct)
    return 

    function cleanup
      if this.hfont<>0
	      deleteobject(this.hfont) // clear the font
   	endif
    return

endclass
